import type { ReactNode } from 'react';
import type { Extension as CodemirrorExtension } from '@codemirror/state';
import { CommonEditor, MarkupString } from '../common';
import { WysiwygEditorOptions } from '../core';
import { ReactRenderStorage } from '../extensions';
import { type CreateCodemirrorParams } from '../markup/codemirror';
import type { YfmLangOptions } from '../markup/codemirror/yfm';
import { Receiver } from '../utils/event-emitter';
import type { FileUploadHandler } from '../utils/upload';
export declare type EditorMode = 'wysiwyg' | 'markup';
export declare type SplitMode = false | 'horizontal' | 'vertical';
export declare type EditorPreset = 'zero' | 'commonmark' | 'default' | 'yfm' | 'full';
export declare type RenderPreview = ({ getValue, mode, }: {
    getValue: () => MarkupString;
    mode: 'preview' | 'split';
}) => ReactNode;
export declare type ToolbarActionData = {
    editorMode: EditorMode;
    id: string;
    attrs?: {
        [key: string]: any;
    };
};
export interface EventMap {
    change: null;
    cancel: null;
    submit: null;
    'toolbar-action': ToolbarActionData;
    'change-editor-mode': {
        mode: EditorMode;
    };
    'change-toolbar-visibility': {
        visible: boolean;
    };
    'change-split-mode-enabled': {
        splitModeEnabled: boolean;
    };
}
export interface Editor extends Receiver<EventMap>, CommonEditor {
    readonly currentMode: EditorMode;
    readonly toolbarVisible: boolean;
    setEditorMode(mode: EditorMode, opts?: SetEditorModeOptions): void;
    moveCursor(position: 'start' | 'end' | {
        line: number;
    }): void;
}
declare type SetEditorModeOptions = Pick<ChangeEditorModeOptions, 'emit'>;
declare type ChangeEditorModeOptions = {
    mode: EditorMode;
    reason: 'error-boundary' | 'settings' | 'manually';
    emit?: boolean;
};
export declare type MarkupConfig = {
    /** Additional extensions for codemirror instance. */
    extensions?: CreateCodemirrorParams['extensions'];
    /** Can be used to disable some of the default extensions */
    disabledExtensions?: CreateCodemirrorParams['disabledExtensions'];
    /** Additional keymaps for codemirror instance */
    keymaps?: CreateCodemirrorParams['keymaps'];
    /** Overrides the default placeholder content. */
    placeholder?: CreateCodemirrorParams['placeholder'];
    /**
     * Additional language data for markdown language in codemirror.
     * Can be used to configure additional autocompletions and others.
     * See more https://codemirror.net/docs/ref/#state.EditorState.languageDataAt
     */
    languageData?: YfmLangOptions['languageData'];
    /** Config for @codemirror/autocomplete https://codemirror.net/docs/ref/#autocomplete.autocompletion%5Econfig */
    autocompletion?: CreateCodemirrorParams['autocompletion'];
};
export declare type EscapeConfig = {
    commonEscape?: RegExp;
    startOfLineEscape?: RegExp;
};
export declare type EditorOptions = Pick<WysiwygEditorOptions, 'allowHTML' | 'linkify' | 'linkifyTlds' | 'extensions'> & {
    initialMarkup?: MarkupString;
    /** @default 'wysiwyg' */
    initialEditorMode?: EditorMode;
    /** @default true */
    initialToolbarVisible?: boolean;
    /** @default false
     * Has no effect if splitMode is false or undefined
     */
    initialSplitModeEnabled?: boolean;
    renderStorage: ReactRenderStorage;
    fileUploadHandler?: FileUploadHandler;
    /**
     * If we need to set dimensions for uploaded images
     *
     * @default false
     */
    needToSetDimensionsForUploadedImages?: boolean;
    /**
     * Called before switching from the markup editor to the wysiwyg editor.
     * You can use it to pre-process the value from the markup editor before it gets into the wysiwyg editor.
     */
    prepareRawMarkup?: (value: MarkupString) => MarkupString;
    experimental_beforeEditorModeChange?: (options: Pick<ChangeEditorModeOptions, 'mode' | 'reason'>) => boolean | undefined;
    splitMode?: SplitMode;
    renderPreview?: RenderPreview;
    preset: EditorPreset;
    /** @deprecated Put extra extensions via MarkdownEditorMarkupConfig */
    extraMarkupExtensions?: CodemirrorExtension[];
    markupConfig?: MarkupConfig;
    escapeConfig?: EscapeConfig;
};
export {};
