"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MarkdownParser = void 0;
const prosemirror_model_1 = require("prosemirror-model");
const logger_1 = require("../../logger");
const openSuffix = '_open';
const closeSuffix = '_close';
var TokenType;
(function (TokenType) {
    TokenType["open"] = "open";
    TokenType["close"] = "close";
    TokenType["default"] = "default";
})(TokenType || (TokenType = {}));
class MarkdownParser {
    constructor(schema, tokenizer, tokens) {
        this.stack = [];
        this.schema = schema;
        this.marks = prosemirror_model_1.Mark.none;
        this.tokens = tokens;
        this.tokenizer = tokenizer;
    }
    validateLink(url) {
        return this.tokenizer.validateLink(url);
    }
    normalizeLink(url) {
        return this.tokenizer.normalizeLink(url);
    }
    normalizeLinkText(url) {
        return this.tokenizer.normalizeLinkText(url);
    }
    matchLinks(text) {
        return this.tokenizer.linkify.match(text);
    }
    parse(text) {
        const time = Date.now();
        try {
            this.stack = [{ type: this.schema.topNodeType, content: [] }];
            let mdItTokens;
            try {
                mdItTokens = this.tokenizer.parse(text, {});
            }
            catch (err) {
                const e = err;
                e.message = 'Unable to parse your markup. Please check for errors. ' + e.message;
                throw e;
            }
            this.parseTokens(mdItTokens);
            let doc;
            // If after parsing there are still unclosed nodes, close them by removing them from the stack.
            do {
                doc = this.closeNode();
            } while (this.stack.length);
            return (doc || this.schema.topNodeType.createAndFill());
        }
        finally {
            logger_1.logger.metrics({ component: 'parser', event: 'parse', duration: Date.now() - time });
        }
    }
    top() {
        return this.stack[this.stack.length - 1];
    }
    push(elt) {
        if (this.stack.length)
            this.top().content.push(elt);
    }
    //#region helpers
    getTokenAttrs(token, tokenSpec, tokenStream, i) {
        if (tokenSpec.getAttrs)
            return tokenSpec.getAttrs(token, tokenStream, i);
        else if (tokenSpec.attrs instanceof Function)
            return tokenSpec.attrs(token);
        return tokenSpec.attrs;
    }
    getTokenSpec(token) {
        var _a;
        // Matching the pm-node meta-attribute which is set in the plugin for markdown-it
        let tokName = ((_a = token.meta) === null || _a === void 0 ? void 0 : _a['pm-node']) || token.type;
        // Cropping _open and _close from node name end
        // e.g. paragraph_open -> paragraph
        tokName = tokName.replace(new RegExp(`(${openSuffix})$|(${closeSuffix})$`), '');
        let tokenSpec;
        if (tokName in this.tokens)
            tokenSpec = this.tokens[tokName];
        if (!tokenSpec)
            throw new RangeError(`No token spec for token: ${JSON.stringify(token)}`);
        return tokenSpec;
    }
    // Getting token type from its name
    // e.g. "paragraph_open" - is opening token for paragraph
    getTokenType({ type }) {
        if (type.endsWith(openSuffix))
            return TokenType.open;
        if (type.endsWith(closeSuffix))
            return TokenType.close;
        return TokenType.default;
    }
    //#endregion
    //#region token handlers
    // Self-explanatory
    handlePrimitiveToken(token) {
        switch (token.type) {
            case 'text':
                this.addText(withoutTrailingNewline(token.content));
                return true;
            case 'inline':
                this.parseTokens(token.children || []);
                return true;
            case 'softbreak':
                // TODO: move all primitive token's handlers to extensions
                if (this.tokens.softbreak)
                    return false;
                this.addText('\n');
                return true;
        }
        return false;
    }
    handleMark(token, tokenSpec, attrs) {
        const schemaSpec = this.schema.marks[tokenSpec.name];
        if (tokenSpec.noCloseToken) {
            this.openMark(schemaSpec.create(attrs));
            let { content } = token;
            if (!tokenSpec.code)
                content = withoutTrailingNewline(content);
            this.addText(content);
            this.closeMark(schemaSpec);
            return;
        }
        switch (this.getTokenType(token)) {
            case 'open': {
                this.openMark(schemaSpec.create(attrs));
                break;
            }
            case 'close': {
                this.closeMark(schemaSpec);
                break;
            }
        }
    }
    handleNode(_token, tokenSpec, attrs) {
        const schemaSpec = this.schema.nodes[tokenSpec.name];
        // Adding node as is, becasuse it doesn't contain content.
        this.addNode(schemaSpec, attrs);
    }
    handleBlock(token, tokenSpec, attrs) {
        var _a;
        const schemaSpec = this.schema.nodes[tokenSpec.name];
        if (tokenSpec.noCloseToken) {
            this.openNode(schemaSpec, attrs);
            if (tokenSpec.contentField === 'children' && ((_a = token.children) === null || _a === void 0 ? void 0 : _a.length)) {
                this.parseTokens(token.children);
            }
            else {
                let { content } = token;
                if (tokenSpec.prepareContent) {
                    content = tokenSpec.prepareContent(content);
                }
                this.addText(content);
            }
            this.closeNode();
            return;
        }
        switch (this.getTokenType(token)) {
            case 'open': {
                this.openNode(schemaSpec, attrs);
                break;
            }
            case 'close': {
                this.closeNode();
                break;
            }
        }
    }
    handleToken(token, tokenSpec, attrs) {
        switch (tokenSpec.type) {
            case 'mark':
                this.handleMark(token, tokenSpec, attrs);
                break;
            case 'node':
                this.handleNode(token, tokenSpec, attrs);
                break;
            case 'block':
                this.handleBlock(token, tokenSpec, attrs);
                break;
        }
    }
    //#endregion
    //#region state methods
    addNode(type, attrs, content) {
        const node = type.createAndFill(attrs, content, this.marks);
        if (!node)
            return null;
        this.push(node);
        return node;
    }
    openNode(type, attrs) {
        this.stack.push({ type: type, attrs, content: [] });
    }
    closeNode() {
        // Marks operate within a node. Therefore, when we close the node, we reset the existing marks.
        if (this.marks.length)
            this.marks = prosemirror_model_1.Mark.none;
        const info = this.stack.pop();
        if (info)
            return this.addNode(info.type, info.attrs, info.content);
        return null;
    }
    addText(text) {
        if (!text)
            return;
        const nodes = this.top().content;
        const last = nodes[nodes.length - 1];
        const node = this.schema.text(text, this.marks);
        let merged;
        if (last && (merged = maybeMerge(last, node)))
            nodes[nodes.length - 1] = merged;
        else
            nodes.push(node);
    }
    // Adding mark to a set of open marks
    openMark(mark) {
        this.marks = mark.addToSet(this.marks);
    }
    closeMark(mark) {
        this.marks = mark.removeFromSet(this.marks);
    }
    //#endregion
    parseTokens(tokens) {
        for (let i = 0; i < tokens.length; i++) {
            const token = tokens[i];
            if (this.handlePrimitiveToken(token))
                continue;
            const tokenSpec = this.getTokenSpec(token);
            if (tokenSpec.ignore)
                continue;
            const attrs = this.getTokenAttrs(token, tokenSpec, tokens, i);
            this.handleToken(token, tokenSpec, attrs);
        }
    }
}
exports.MarkdownParser = MarkdownParser;
// Checking if these are two text nodes and they have the same marks, then it merges them into one.
function maybeMerge(a, b) {
    if (a.isText && b.isText && prosemirror_model_1.Mark.sameSet(a.marks, b.marks)) {
        // Somehow this method is not in typings but it exists.
        // @ts-expect-error
        return a.withText(a.text + b.text);
    }
}
function withoutTrailingNewline(str) {
    return str[str.length - 1] === '\n' || str.endsWith('\\n') ? str.slice(0, str.length - 1) : str;
}
