"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ImagePaste = void 0;
const base64_arraybuffer_1 = require("base64-arraybuffer");
const prosemirror_model_1 = require("prosemirror-model");
const prosemirror_state_1 = require("prosemirror-state");
const prosemirror_transform_1 = require("prosemirror-transform");
const lodash_1 = require("../../../../lodash");
const Clipboard_1 = require("../../../behavior/Clipboard");
const specs_1 = require("../../../specs");
const utils_1 = require("../utils");
const upload_1 = require("./upload");
const { isFilesFromHtml, isFilesOnly, isImageFile } = Clipboard_1.clipboardUtils;
const ImagePaste = (builder, opts) => {
    if (!opts || !(0, lodash_1.isFunction)(opts.imageUploadHandler))
        throw new Error('ImagePaste extension: imageUploadHandler is not a function');
    builder.addPlugin(() => new prosemirror_state_1.Plugin({
        props: {
            handleDOMEvents: {
                paste(view, e) {
                    const files = getPastedImages(e.clipboardData);
                    if (files) {
                        e.preventDefault();
                        new upload_1.ImagesUploadProcess(view, files, opts.imageUploadHandler, view.state.tr.selection.from, opts).run();
                        return true;
                    }
                    return false;
                },
                drop(view, e) {
                    var _a, _b;
                    // handle drop images from device
                    if (view.dragging)
                        return false;
                    const files = getPastedImages(e.dataTransfer);
                    if (!files)
                        return false;
                    const dropPos = (_b = (_a = view.posAtCoords({ left: e.clientX, top: e.clientY })) === null || _a === void 0 ? void 0 : _a.pos) !== null && _b !== void 0 ? _b : -1;
                    if (dropPos === -1)
                        return false;
                    const posToInsert = (0, prosemirror_transform_1.dropPoint)(view.state.doc, dropPos, createFakeImageSlice(view.state.schema));
                    if (posToInsert !== null) {
                        new upload_1.ImagesUploadProcess(view, files, opts.imageUploadHandler, posToInsert, opts).run();
                    }
                    e.preventDefault();
                    return true;
                },
            },
            handlePaste(view, _event, slice) {
                const node = sliceSingleNode(slice);
                if (node && (0, utils_1.isImageNode)(node)) {
                    const imgUrl = node.attrs[specs_1.ImgSizeAttr.Src];
                    const imgFile = dataUrlToFile(imgUrl, 'image');
                    if (imgFile) {
                        new upload_1.ImagesUploadProcess(view, [imgFile], opts.imageUploadHandler, view.state.tr.selection.from, opts).run();
                        return true;
                    }
                }
                return false;
            },
        },
    }), builder.Priority.High);
};
exports.ImagePaste = ImagePaste;
function getPastedImages(data) {
    if (!data || data.files.length === 0)
        return null;
    if (!isFilesOnly(data) && !isFilesFromHtml(data))
        return null;
    const files = Array.from(data.files);
    return files.every(isImageFile) ? files : null;
}
function createFakeImageSlice(schema) {
    return new prosemirror_model_1.Slice(prosemirror_model_1.Fragment.from((0, specs_1.imageType)(schema).create({
        [specs_1.ImageAttr.Src]: 'fake',
        [specs_1.ImageAttr.Title]: 'image',
    })), 0, 0);
}
// copied from prosemirror-view input.ts
function sliceSingleNode(slice) {
    return slice.openStart === 0 && slice.openEnd === 0 && slice.content.childCount === 1
        ? slice.content.firstChild
        : null;
}
function dataUrlToFile(dataUrl, filename) {
    var _a;
    const [data, base64String] = dataUrl.split(',');
    if (!data || !base64String)
        return null;
    const mime = (_a = data.match(/^data:(.+);base64/)) === null || _a === void 0 ? void 0 : _a[1];
    if (!mime)
        return null;
    return new File([(0, base64_arraybuffer_1.decode)(base64String)], filename, { type: mime });
}
